@@ -75,17 +75,30 @@ void svn_swig_py_set_application_pool(PyObject *py_pool, apr_pool_t *pool);
 SVN_SWIG_SWIGUTIL_EXPORT
 void svn_swig_py_clear_application_pool(void);
 
-/* Get the application pool */
+/* Get the pool argument from the last element of tuple args.
+ * If the last element of args is not a pool, create a new
+ * subpool. Return 0 if successful. Return 1 if an error
+ * occurs.
+ */
 SVN_SWIG_SWIGUTIL_EXPORT
-void svn_swig_get_application_pool(PyObject **py_pool, apr_pool_t **pool);
+int svn_swig_py_get_pool_arg(PyObject *args, swig_type_info *type,
+    PyObject **py_pool, apr_pool_t **pool);
+
+/* Get the parent pool of the first argument in the specified
+ * argument list. Return 0 if successful. Return 1 if an error
+ * occurs.
+ */
+SVN_SWIG_SWIGUTIL_EXPORT
+int svn_swig_py_get_parent_pool(PyObject *args, swig_type_info *type,
+    PyObject **py_pool, apr_pool_t **pool);
 
 
 /*** SWIG Wrappers ***/
 
 /* Wrapper for SWIG_NewPointerObj */
 SVN_SWIG_SWIGUTIL_EXPORT
-PyObject *svn_swig_NewPointerObj(void *obj, swig_type_info *type, 
-                                 PyObject *pool);
+PyObject *svn_swig_NewPointerObj(void *obj, swig_type_info *type,
+                                 PyObject *pool, PyObject *args);
 
 /* Wrapper for SWIG_ConvertPtr */
 SVN_SWIG_SWIGUTIL_EXPORT
@@ -93,20 +106,12 @@ int svn_swig_ConvertPtr(PyObject *input, void **obj, swig_type_info *type);
 
 /* Wrapper for SWIG_MustGetPtr */
 SVN_SWIG_SWIGUTIL_EXPORT
-void *svn_swig_MustGetPtr(void *input, swig_type_info *type, int argnum,
-                          PyObject **py_pool);
+void *svn_swig_MustGetPtr(void *input, swig_type_info *type, int argnum);
 
 /*** Functions to expose a custom SubversionException ***/
 
-/* register a new subversion exception class */
-SVN_SWIG_SWIGUTIL_EXPORT
-PyObject *svn_swig_py_register_exception(void);
-
-/* get the object which represents the subversion exception class */
-SVN_SWIG_SWIGUTIL_EXPORT
-PyObject *svn_swig_py_exception_type(void);
-
-/* raise a subversion exception, created from a normal subversion error */
+/* raise a subversion exception, created from a normal subversion
+   error.  consume the error.  */
 SVN_SWIG_SWIGUTIL_EXPORT
 void svn_swig_py_svn_exception(svn_error_t *err);
 
@@ -122,9 +127,36 @@ PyObject *svn_swig_py_prophash_to_dict(apr_hash_t *hash);
 SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_locationhash_to_dict(apr_hash_t *hash);
 
+/* helper function to convert an apr_array_header_t* (of
+   svn_merge_range_t *) to a Python list */
+SVN_SWIG_SWIGUTIL_EXPORT
+PyObject *svn_swig_py_rangelist_to_list(apr_array_header_t *rangelist,
+                                        swig_type_info *type,
+                                        PyObject *py_pool);
+
+/* helper function to convert an apr_hash_t* (const char *->array of
+   svn_merge_range_t *) to a Python dict */
+SVN_SWIG_SWIGUTIL_EXPORT
+PyObject *svn_swig_py_mergeinfo_to_dict(apr_hash_t *hash,
+                                        swig_type_info *type,
+                                        PyObject *py_pool);
+
+/* helper function to convert an apr_hash_t* (const char *->hash of
+   mergeinfo hashes) to a Python dict */
+SVN_SWIG_SWIGUTIL_EXPORT
+PyObject *svn_swig_py_mergeinfo_catalog_to_dict(apr_hash_t *hash,
+                                                swig_type_info *type,
+                                                PyObject *py_pool);
+
+/* helper function to convert an apr_hash_t *(const char *->const char
+ *) to a Python dict */
+
+SVN_SWIG_SWIGUTIL_EXPORT
+PyObject *svn_swig_py_stringhash_to_dict(apr_hash_t *hash);
+
 /* convert a hash of 'const char *' -> TYPE into a Python dict */
 SVN_SWIG_SWIGUTIL_EXPORT
-PyObject *svn_swig_py_convert_hash(apr_hash_t *hash, swig_type_info *type, 
+PyObject *svn_swig_py_convert_hash(apr_hash_t *hash, swig_type_info *type,
                                    PyObject *py_pool);
 
 /* helper function to convert a 'char **' into a Python list of string
@@ -137,11 +169,19 @@ PyObject *svn_swig_py_c_strings_to_list(char **strings);
 SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_array_to_list(const apr_array_header_t *strings);
 
+/* helper function to convert a hash mapping char * to svn_string_t * to a
+ * Python dict mapping str to str. */
+SVN_SWIG_SWIGUTIL_EXPORT
+PyObject *svn_swig_py_changed_path_hash_to_dict(apr_hash_t *hash);
+
+SVN_SWIG_SWIGUTIL_EXPORT
+apr_array_header_t *svn_swig_py_rangelist_to_array(PyObject *list,
+                                                   apr_pool_t *pool);
+
 /* helper function to convert an array of 'svn_revnum_t' to a Python list
    of int objects */
-/* Formerly used by pre-1.0 APIs. Now unused
+SVN_SWIG_SWIGUTIL_EXPORT
 PyObject *svn_swig_py_revarray_to_list(const apr_array_header_t *revs);
-*/
 
 /* helper function to convert a Python dictionary mapping strings to
    strings into an apr_hash_t mapping const char *'s to const char *'s,
@@ -150,6 +190,25 @@ SVN_SWIG_SWIGUTIL_EXPORT
 apr_hash_t *svn_swig_py_stringhash_from_dict(PyObject *dict,
                                              apr_pool_t *pool);
 
+/* helper function to convert a Python dictionary mapping strings to
+   rangelists into an apr_hash_t mapping const char *'s to rangelists,
+   allocated in POOL. */
+SVN_SWIG_SWIGUTIL_EXPORT
+apr_hash_t *svn_swig_py_mergeinfo_from_dict(PyObject *dict,
+                                             apr_pool_t *pool);
+
+/* helper function to convert a Python dictionary mapping strings to
+   strings into an 'apr_array_header_t *' of svn_prop_t *
+   allocated in POOL. */
+SVN_SWIG_SWIGUTIL_EXPORT
+apr_array_header_t *svn_swig_py_proparray_from_dict(PyObject *dict,
+                                                    apr_pool_t *pool);
+
+/* helper function to convert a 'apr_array_header_t *' of 'svn_prop_t
+   to a Python dictionary mapping strings to strings. */
+SVN_SWIG_SWIGUTIL_EXPORT
+PyObject *svn_swig_py_proparray_to_dict(const apr_array_header_t *array);
+
 /* helper function to convert a Python dictionary mapping strings to
    strings into an apr_hash_t mapping const char *'s to svn_string_t's,
    allocated in POOL. */
@@ -157,6 +216,20 @@ SVN_SWIG_SWIGUTIL_EXPORT
 apr_hash_t *svn_swig_py_prophash_from_dict(PyObject *dict,
                                            apr_pool_t *pool);
 
+/* helper function to convert a Python dictionary mapping strings to
+   integers into an apr_hash_t mapping const char *'s to revnums,
+   allocated in POOL. */
+SVN_SWIG_SWIGUTIL_EXPORT
+apr_hash_t *svn_swig_py_path_revs_hash_from_dict(PyObject *dict,
+                                                 apr_pool_t *pool);
+
+/* helper function to convert a Python dictionary mapping strings to
+   strings into an apr_hash_t mapping const char *'s to svn_string_t *'s,
+   allocated in POOL. */
+SVN_SWIG_SWIGUTIL_EXPORT
+apr_hash_t *svn_swig_py_changed_path_hash_from_dict(PyObject *dict,
+                                                    apr_pool_t *pool);
+
 /* helper function to convert a Python sequence of strings into an
    'apr_array_header_t *' of 'const char *' objects.  Note that the
    objects must remain alive -- the values are not copied. This is
@@ -198,6 +271,11 @@ void svn_swig_py_notify_func(void *baton,
                              svn_wc_notify_state_t prop_state,
                              svn_revnum_t revision);
 
+SVN_SWIG_SWIGUTIL_EXPORT
+void svn_swig_py_notify_func2(void *baton,
+                              const svn_wc_notify_t *notify,
+                              apr_pool_t *pool);
+
 /* a status function that executes a Python function that is passed in
    via the baton argument */
 SVN_SWIG_SWIGUTIL_EXPORT
@@ -205,6 +283,22 @@ void svn_swig_py_status_func(void *baton,
                              const char *path,
                              svn_wc_status_t *status);
 
+/* a svn_delta_path_driver callback that executes a Python function
+  that is passed in via the baton argument */
+SVN_SWIG_SWIGUTIL_EXPORT
+svn_error_t *svn_swig_py_delta_path_driver_cb_func(void **dir_baton,
+                                                   void *parent_baton,
+                                                   void *callback_baton,
+                                                   const char *path,
+                                                   apr_pool_t *pool);
+
+/* a status function that executes a Python function that is passed in
+   via the baton argument */
+SVN_SWIG_SWIGUTIL_EXPORT
+void svn_swig_py_status_func2(void *baton,
+                              const char *path,
+                              svn_wc_status2_t *status);
+
 /* a cancel function that executes a Python function passed in via the
    cancel_baton argument. */
 SVN_SWIG_SWIGUTIL_EXPORT
@@ -212,15 +306,16 @@ svn_error_t *svn_swig_py_cancel_func(void *cancel_baton);
 
 /* thunked fs get_locks function */
 SVN_SWIG_SWIGUTIL_EXPORT
-svn_error_t *svn_swig_py_fs_get_locks_func(void *baton, 
-                                           svn_lock_t *lock, 
+svn_error_t *svn_swig_py_fs_get_locks_func(void *baton,
+                                           svn_lock_t *lock,
                                            apr_pool_t *pool);
 
 /* thunked commit log fetcher */
 SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_get_commit_log_func(const char **log_msg,
                                              const char **tmp_file,
-                                             apr_array_header_t *commit_items,
+                                             const apr_array_header_t *
+                                             commit_items,
                                              void *baton,
                                              apr_pool_t *pool);
 
@@ -249,6 +344,12 @@ svn_error_t *svn_swig_py_log_receiver(void *py_receiver,
                                       const char *msg,
                                       apr_pool_t *pool);
 
+/* thunked log receiver2 function */
+SVN_SWIG_SWIGUTIL_EXPORT
+svn_error_t *svn_swig_py_log_entry_receiver(void *baton,
+                                            svn_log_entry_t *log_entry,
+                                            apr_pool_t *pool);
+
 /* thunked info receiver function */
 SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_info_receiver_func(void *py_receiver,
@@ -256,6 +357,13 @@ svn_error_t *svn_swig_py_info_receiver_func(void *py_receiver,
                                             const svn_info_t *info,
                                             apr_pool_t *pool);
 
+/* thunked location segments receiver function */
+SVN_SWIG_SWIGUTIL_EXPORT
+svn_error_t *
+svn_swig_py_location_segment_receiver_func(svn_location_segment_t *segment,
+                                           void *baton,
+                                           apr_pool_t *pool);
+
 /* thunked blame receiver function */
 SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_client_blame_receiver_func(void *baton,
@@ -266,6 +374,13 @@ svn_error_t *svn_swig_py_client_blame_receiver_func(void *baton,
                                                     const char *line,
                                                     apr_pool_t *pool);
 
+/* thunked changelist receiver function */
+SVN_SWIG_SWIGUTIL_EXPORT
+svn_error_t *svn_swig_py_changelist_receiver_func(void *baton,
+                                                  const char *path,
+                                                  const char *changelist,
+                                                  apr_pool_t *pool);
+
 /* auth provider callbacks */
 SVN_SWIG_SWIGUTIL_EXPORT
 svn_error_t *svn_swig_py_auth_simple_prompt_func(
@@ -310,6 +425,54 @@ svn_error_t *svn_swig_py_auth_ssl_client_cert_pw_prompt_func(
     svn_boolean_t may_save,
     apr_pool_t *pool);
 
+SVN_SWIG_SWIGUTIL_EXPORT
+void
+svn_swig_py_setup_ra_callbacks(svn_ra_callbacks2_t **callbacks,
+                               void **baton,
+                               PyObject *py_callbacks,
+                               apr_pool_t *pool);
+
+SVN_SWIG_SWIGUTIL_EXPORT
+svn_wc_diff_callbacks2_t *
+svn_swig_py_setup_wc_diff_callbacks2(void **baton,
+                                     PyObject *py_callbacks,
+                                     apr_pool_t *pool);
+
+SVN_SWIG_SWIGUTIL_EXPORT
+svn_error_t *svn_swig_py_commit_callback2(const svn_commit_info_t *commit_info,
+                                          void *baton,
+                                          apr_pool_t *pool);
+
+SVN_SWIG_SWIGUTIL_EXPORT
+svn_error_t *svn_swig_py_commit_callback(svn_revnum_t new_revision,
+                                         const char *date,
+                                         const char *author,
+                                         void *baton);
+
+
+SVN_SWIG_SWIGUTIL_EXPORT
+svn_error_t *svn_swig_py_ra_file_rev_handler_func(
+                    void *baton,
+                    const char *path,
+                    svn_revnum_t rev,
+                    apr_hash_t *rev_props,
+                    svn_txdelta_window_handler_t *delta_handler,
+                    void **delta_baton,
+                    apr_array_header_t *prop_diffs,
+                    apr_pool_t *pool);
+
+SVN_SWIG_SWIGUTIL_EXPORT
+svn_error_t *svn_swig_py_ra_lock_callback(
+                    void *baton,
+                    const char *path,
+                    svn_boolean_t do_lock,
+                    const svn_lock_t *lock,
+                    svn_error_t *ra_err,
+                    apr_pool_t *pool);
+
+SVN_SWIG_SWIGUTIL_EXPORT
+extern const svn_ra_reporter2_t swig_py_ra_reporter2;
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
