@@ -1,7 +1,7 @@
 /* fs.h : interface to Subversion filesystem, private to libsvn_fs
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -33,15 +33,34 @@ extern "C" {
 #endif /* __cplusplus */
 
 
-/*** The filesystem structure.  ***/
+/*** Filesystem schema versions ***/
 
-/* The format number of this filesystem.
-   This is independent of the repository format number, and
-   independent of any other FS back ends. */
-#define SVN_FS_BASE__FORMAT_NUMBER   2
+/* The format number of this filesystem.  This is independent of the
+   repository format number, and independent of any other FS back
+   ends.  See the SVN_FS_BASE__MIN_*_FORMAT defines to get a sense of
+   what changes and features were added in which versions of this
+   back-end's format.  */
+#define SVN_FS_BASE__FORMAT_NUMBER                3
+
+/* Minimum format number that supports node-origins tracking */
+#define SVN_FS_BASE__MIN_NODE_ORIGINS_FORMAT      3
+
+/* Minimum format number that supports mergeinfo */
+#define SVN_FS_BASE__MIN_MERGEINFO_FORMAT         3
 
 /* Minimum format number that supports svndiff version 1.  */
-#define SVN_FS_BASE__MIN_SVNDIFF1_FORMAT 2
+#define SVN_FS_BASE__MIN_SVNDIFF1_FORMAT          2
+
+/* Return SVN_ERR_UNSUPPORTED_FEATURE if the version of filesystem FS does
+   not indicate support for FEATURE (which REQUIRES a newer version). */
+svn_error_t *
+svn_fs_base__test_required_feature_format(svn_fs_t *fs, 
+                                          const char *feature, 
+                                          int requires);
+
+
+
+/*** The filesystem structure.  ***/
 
 typedef struct
 {
@@ -60,6 +79,7 @@ typedef struct
   DB *uuids;
   DB *locks;
   DB *lock_tokens;
+  DB *node_origins;
 
   /* A boolean for tracking when we have a live Berkeley DB
      transaction trail alive. */
@@ -74,18 +94,6 @@ typedef struct
 } base_fs_data_t;
 
 
-/* Return a canonicalized version of a filesystem PATH, allocated in
-   POOL.  While the filesystem API is pretty flexible about the
-   incoming paths (they must be UTF-8 with '/' as separators, but they
-   don't have to begin with '/', and multiple contiguous '/'s are
-   ignored) we want any paths that are physically stored in the
-   underlying database to look consistent.  Specifically, absolute
-   filesystem paths should begin with '/', and all redundant and trailing '/'
-   characters be removed.  */
-const char *
-svn_fs_base__canonicalize_abspath(const char *path, apr_pool_t *pool);
-
-
 /*** Filesystem Revision ***/
 typedef struct
 {
@@ -164,6 +172,14 @@ typedef struct
   /* path at which this node first came into existence.  */
   const char *created_path;
 
+  /* does this node revision have the mergeinfo tracking property set
+     on it?  (only valid for FS schema 3 and newer) */
+  svn_boolean_t has_mergeinfo;
+
+  /* number of children of this node which have the mergeinfo tracking
+     property set  (0 for files; valid only for FS schema 3 and newer). */
+  apr_int64_t mergeinfo_count;
+
 } node_revision_t;
 
 
@@ -192,7 +208,7 @@ typedef struct
   /* size of the fulltext data represented by this delta window. */
   apr_size_t size;
 
-  /* represenatation-key to use when needed source data for
+  /* representation-key to use when needed source data for
      undeltification. */
   const char *rep_key;
 
